__uname__ = 'sqjobs-tf'
__long_name__ = 'Simple Queue Jobs TF Fork'
__version__ = '0.8.7'
__author__ = 'Thinkful'
__email__ = 'engineering@thinkful.com'
__url__ = 'http://github.com/Thinkful/sqjobs'
__license__ = 'BSD'
